#
#  Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.
#
#  File:    gco1.py
#
#  Purpose: Demonstrates how to use the General Convex solver in
#           MOSEK. The example has non-linear convex objective.
##

import sys

import pymosek as mosek

from math import exp
import Numeric
from Numeric import array, Float, zeros, ones

def sqr(v): return v*v

# Since the actual value of Infinity is ignores, we define it solely
# for symbolic purposes:
inf = 0.0

# Define a stream printer to grab output from MOSEK
def streamprinter(text):
    sys.stdout.write(text)
    sys.stdout.flush()
 
class ConvexObjective:
    """
    Provide functions to evaluate:

    exp((x-a)^2 + (y-b)^2)
    """
    def __init__ (self,a,b):
        assert a > 0 and b > 0
        
        self.a = a
        self.b = b


    def numobjnz(self):
        """
        Number of non-linear non-zeros in objective.
        """
        return 2
    def maxnumhesnz(self):
        """
        Number of non-zeros in the full hessian of the lagrangian.
        """
        return 4
    def congrdsub(self):
        """
        Indexes of non-linear constraints. The are 0.
        """
        return None
    def congrdnnz(self):
        """
        For each row, the number of non-linear non-zeros. Again, there
        are no non-linear constraints.
        """
        return None

    ##################################################
    # Evaluation functions
    
    def objval(self,xx):
        """
        Compute objective value
        """
        objval = exp(sqr(xx[0]-self.a) + sqr(xx[0]-self.b))
        return objval
    
    def objgrd(self,xx,sub,val):
        """
        Compute gradient of objective
        """

        if sub is not None:
            sub[0] = 0
            sub[1] = 1

        sys.stdout.flush()
        if val is not None:
            m = exp(sqr(xx[0]-self.a) + sqr(xx[1]-self.b))
            val[0] = m * 2 * (xx[0] - self.a)
            val[1] = m * 2 * (xx[1] - self.b)

    def hessian(self,xx,yo,subi,yc,hessubi,hessubj,hesval):
        """
        Compute hessian of lagrangian.

        Since only the objective is non-linear, we ignore subi and yc.
        """
        if (hessubi is not None and
            hessubj is not None):
            hessubi[0] = 0
            hessubi[1] = 1
            hessubi[2] = 0
            hessubi[3] = 1

            hessubj[0] = 0
            hessubj[1] = 0
            hessubj[2] = 1
            hessubj[3] = 1

        if hesval is not None:
            m = exp(sqr(xx[0]-self.a) + sqr(xx[1]-self.b))
            
            hesval[0] = yo * (4 * m * sqr(xx[0]-self.a) + 2 * m)
            hesval[1] = yo * (4 * m * (xx[0]-self.a) * (xx[1]-self.b))
            hesval[2] = yo * (4 * m * (xx[0]-self.a) * (xx[1]-self.b))
            hesval[3] = yo * (4 * m * sqr(xx[1]-self.b) + 2 * m)


        return 4
    

            


# We might write everything directly as a script, but it looks nicer
# to create a function.
def main ():
    # Create a handle to MOSEK
    mskhandle = mosek.mosek ()
    # Make a MOSEK environment
    env = mskhandle.Env ()
    # Attach a printer to the environment
    env.set_Stream (mosek.streamtype.log, streamprinter)
    # Initialize the environment
    env.init ()

    # Create a task
    task = env.Task(0,0)
    # Attach a printer to the task
    task.set_Stream (mosek.streamtype.log, streamprinter)


    # append variables and constraints
    numvar = 2
    task.append(mosek.accmode.var, numvar)
    
    task.putbound(mosek.accmode.var, 0, mosek.boundkey.ra, 0.0, 2.0)
    task.putbound(mosek.accmode.var, 1, mosek.boundkey.ra, 1.0, 3.0)
    
    # input the non-linear objective

    gco = ConvexObjective(1.0, 2.0)
    

    task.putGCeval(gco.numobjnz(),
                   gco.congrdsub(),
                   gco.congrdnnz(),
                   gco.maxnumhesnz(),

                   getobjval  = gco.objval,
                   getobjgrd  = gco.objgrd,
                   #getconvali, 
                   #getconigrd,
                   gethes     = gco.hessian)

    # Input the objective sense (minimize/maximize)
    task.putobjsense(mosek.objsense.minimize)
    
    
    # Optimize the task
    task.optimize();

    # Output a solution
    xx = zeros(numvar, Float)
    task.getsolutionslice(mosek.soltype.itr,
                          mosek.solitem.xx, 
                          0,numvar,          
                          xx)
    print "xx =", xx



# call the main function
try:
    main ()
except mosek.Exception, err:
    print "ERROR: %s" % str(err.errno)
    if err.msg is not None:
        print "\t%s" % err.msg
        
    sys.exit(1)
except:
    import traceback
    traceback.print_exc()
    sys.exit(1)
