#
#  Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.
#
#  File:    lo2.py
#
#  Purpose: Demonstrates how to solve small linear
#           optimization problem using the MOSEK Python API.
##

import sys

import pymosek as mosek

from mosekarr import array, Float, zeros, ones

# Since the actual value of Infinity is ignores, we define it solely
# for symbolic purposes:
inf = 0.0

# Define a stream printer to grab output from MOSEK
def streamprinter(text):
    sys.stdout.write(text)
    sys.stdout.flush()


# We might write everything directly as a script, but it looks nicer
# to create a function.
def main ():
    # Create a handle to MOSEK
    mskhandle = mosek.mosek ()
    # Make a MOSEK environment
    env = mskhandle.Env ()
    # Attach a printer to the environment
    env.set_Stream (mosek.streamtype.log, streamprinter)
    # Initialize the environment
    env.init ()

    # Create a task
    task = env.Task(0,0)
    # Attach a printer to the task
    task.set_Stream (mosek.streamtype.log, streamprinter)


    # Bound keys for constraints
    bkc = array ([mosek.boundkey.fx,
                  mosek.boundkey.lo,
                  mosek.boundkey.up])
    # Bound values for constraints
    blc = array ([30.0, 15.0, -inf])
    buc = array ([30.0, +inf, 25.0])
    
    # Bound keys for variables
    bkx = array ([mosek.boundkey.lo,
                  mosek.boundkey.ra,
                  mosek.boundkey.lo,
                  mosek.boundkey.lo])
    # Bound values for variables
    blx = array ([ 0.0,  0.0,  0.0,  0.0])
    bux = array ([+inf, 10.0, +inf, +inf])

    # Objective coefficients
    csub = array([   0,   1,   2,   3 ])
    cval = array([ 3.0, 1.0, 5.0, 1.0 ])

    # We input the A matrix column-wise
    # asub contains row indexes
    asub = array([ 0, 1,
                   0, 1, 2,
                   0, 1,
                      1, 2])
    # acof contains coefficients
    acof = array([ 3.0, 2.0,
                   1.0, 1.0, 2.0,
                   2.0, 3.0,
                        1.0, 3.0 ])
    # aptrb and aptre contains the offsets into asub and acof where
    # columns start and end respectively
    aptrb = array([ 0, 2, 5, 7 ])
    aptre = array([ 2, 5, 7, 9 ])

    numvar = len(bkx)
    numcon = len(bkc)

    # Append the constraints
##REF:func:task:append
##REF:enum:accmode:con
    task.append(mosek.accmode.con,numcon);
    
    # Append the variables.
##REF:func:task:append
##REF:enum:accmode:var
    task.append(mosek.accmode.var,numvar);
    
    # Input objective
##REF:func:task:putcfix
    task.putcfix(0.0)
##REF:func:task:putclist
    task.putclist(csub,cval);

    # Put constraint bounds
##REF:func:task:putboundslice
    task.putboundslice(mosek.accmode.con,
                       0, numcon,
                       bkc, blc, buc);

    # Put variable bounds
##REF:func:task:putboundslice
    task.putboundslice(mosek.accmode.var,
                       0, numvar,
                       bkx, blx, bux);
    


    # Input A non-zeros by columns
##REF:func:task:putavec
    for j in range(numvar):
        ptrb,ptre = aptrb[j],aptre[j]
        task.putavec(mosek.accmode.var,j,
                     asub[ptrb:ptre],
                     acof[ptrb:ptre])
                   
    # Input the objective sense (minimize/maximize)
    task.putobjsense(mosek.objsense.maximize)
    
    
    # Optimize the task
    task.optimize();

    # Output a solution
    xx = zeros(numvar, Float)
    task.getsolutionslice(mosek.soltype.bas,
                          mosek.solitem.xx, 
                          0,numvar,          
                          xx)
    print "xx =", xx



# call the main function
try:
    main ()
except mosek.Exception, (code,msg):
    print "ERROR: %s" % str(code)
    if msg is not None:
        print "\t%s" % msg
    sys.exit(1)
except:
    import traceback
    traceback.print_exc()
    sys.exit(1)
