##
#    Copyright: Copyright (c) 1998-2007 MOSEK ApS, Denmark. All rights reserved.
#
#    File:    mio1.cs
#
#    Purpose:  Demonstrates how to solve a small mixed
#              integer linear optimization problem using the MOSEK Python API.
##

import sys

import pymosek as mosek

from mosekarr import array, Float, zeros, ones

# Since the actual value of Infinity is ignores, we define it solely
# for symbolic purposes:
inf = 0.0


# Define a stream printer to grab output from MOSEK
def streamprinter(text):
    sys.stdout.write(text)
    sys.stdout.flush()


# We might write everything directly as a script, but it looks nicer
# to create a function.
def main ():
    # Create a handle to MOSEK
    mskhandle = mosek.mosek ()
    # Make a MOSEK environment
    env = mskhandle.Env ()
    # Attach a printer to the environment
    env.set_Stream (mosek.streamtype.log, streamprinter)
    # Initialize the environment
    env.init ()

    # Create a task
    task = env.Task(0,0)
    # Attach a printer to the task
    task.set_Stream (mosek.streamtype.log, streamprinter)

    
    bkc = array([ mosek.boundkey.up, mosek.boundkey.lo  ])
    blc = array([              -inf,              -4.0  ])
    buc = array([             250.0,               inf  ]) 

    bkx = array([ mosek.boundkey.lo, mosek.boundkey.lo  ])
    blx = array([               0.0,               0.0  ])
    bux = array([               inf,               inf  ])

    c   = array([               1.0,               0.64 ])

    asub = array([  0,   1,    0,    1   ])
    acof = array([ 50.0, 3.0, 31.0, -2.0 ])

    ptrb = array([ 0, 2 ])
    ptre = array([ 2, 4 ])

    numvar = len(bkx)
    numcon = len(bkc)

    # Input linear data
    task.inputdata(numcon,numvar,
                   c,0.0,
                   ptrb, ptre, asub, acof,
                   bkc,  blc,  buc,
                   bkx,  blx,  bux)
    
    # Input objective sense
    task.putobjsense(mosek.objsense.maximize)

    # Define variables to be integers
##REF:func:task:putvartypelist
    task.putvartypelist(array([ 0, 1 ]),
                        array([ mosek.variabletype.type_int,
                                mosek.variabletype.type_int ]))
    task.putintparam (mosek.iparam.mio_mode,
                      mosek.miomode.satisfied)
    # Optimize
    task.optimize()

##REF:func:task:solutiondef
##REF:enum:soltype:itg
    if task.solutiondef(mosek.soltype.itg):

        # Output a solution
        xx = zeros(numvar, Float)
        task.getsolutionslice(mosek.soltype.itg,
                              mosek.solitem.xx,
                              0, numvar,
                              xx)
        print "x =", xx
    else:
        print "Integer solution not defined. Probably a problem with 'mosekglb' optimizer."

# call the main function
try:
    main ()
except mosek.Exception, e:
    print "ERROR: %s" % str(e.errno)
    if e.msg is not None:
        print "\t%s" % e.msg
    sys.exit(1)
except:
    import traceback
    traceback.print_exc()
    sys.exit(1)

